import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.preprocessing import StandardScaler, MinMaxScaler

# 1. サンプルデータの作成
np.random.seed(42)
data = {
    "CustomerID": np.random.randint(1000, 2000, 200),
    "TransactionDate": pd.date_range("2023-01-01", periods=200, freq="D"),
    "PurchaseAmount": np.random.uniform(100, 1000, 200).round(2),
    "ProductCategory": np.random.choice(["Electronics", "Clothing", "Groceries"], 200, p=[0.4, 0.3, 0.3]),
    "Discount": np.random.randint(0, 50, 200),
    "Quantity": np.random.randint(1, 10, 200)
}
df = pd.DataFrame(data)

# 2. データの概要確認
print("データの概要:")
print(df.info())
print("\nデータの統計量:")
print(df.describe())

# 3. 欠損値処理（例: 欠損値をランダムに挿入し補完）
df.loc[np.random.choice(df.index, 20, replace=False), "PurchaseAmount"] = np.nan
print("\n欠損値を挿入:")
print(df.isnull().sum())
df["PurchaseAmount"] = df["PurchaseAmount"].fillna(df["PurchaseAmount"].median())

# 4. 異常値処理（IQRを使用）
Q1 = df["PurchaseAmount"].quantile(0.25)
Q3 = df["PurchaseAmount"].quantile(0.75)
IQR = Q3 - Q1
lower_bound = Q1 - 1.5 * IQR
upper_bound = Q3 + 1.5 * IQR
df["PurchaseAmount"] = np.where(
    df["PurchaseAmount"] > upper_bound, upper_bound, 
    np.where(df["PurchaseAmount"] < lower_bound, lower_bound, df["PurchaseAmount"])
)

# 5. カテゴリ変数のエンコーディング
df_encoded = pd.get_dummies(df, columns=["ProductCategory"], drop_first=True)

# 6. スケーリング
scaler = StandardScaler()
df_encoded["PurchaseAmount_Scaled"] = scaler.fit_transform(df_encoded[["PurchaseAmount"]])

# 7. 日付データの分解
df_encoded["TransactionMonth"] = df["TransactionDate"].dt.month
df_encoded["TransactionDayOfWeek"] = df["TransactionDate"].dt.dayofweek

# 8. 可視化
# ヒストグラム: 購入金額の分布
plt.figure(figsize=(8, 6))
sns.histplot(df_encoded["PurchaseAmount"], bins=20, kde=True, color="blue")
plt.title("購入金額の分布")
plt.xlabel("購入金額")
plt.ylabel("頻度")
plt.show()

# 棒グラフ: カテゴリ別の平均購入金額
plt.figure(figsize=(8, 6))
sns.barplot(x="ProductCategory", y="PurchaseAmount", data=df, ci=None, palette="pastel")
plt.title("カテゴリ別の平均購入金額")
plt.xlabel("カテゴリ")
plt.ylabel("平均購入金額")
plt.show()

# 散布図: 購入金額 vs 割引率
plt.figure(figsize=(8, 6))
sns.scatterplot(x="Discount", y="PurchaseAmount", hue="Quantity", size="Quantity", data=df, palette="viridis")
plt.title("購入金額と割引率の関係")
plt.xlabel("割引率")
plt.ylabel("購入金額")
plt.legend(title="購入数量", bbox_to_anchor=(1.05, 1), loc="upper left")
plt.tight_layout()
plt.show()

# 9. 派生特徴量の作成（例: 顧客ごとの累積購入金額）
df_encoded["CumulativePurchase"] = df.groupby("CustomerID")["PurchaseAmount"].cumsum()

# 10. 最終データの表示
print("\n最終データのプレビュー:")
print(df_encoded.head())
